<el-slider
  style="padding: 20px"
  v-model="state.{{{widget.widget_id}}}.value"
  :min="data.{{{widget.widget_id}}}.min"
  :max="data.{{{widget.widget_id}}}.max"
  :step="data.{{{widget.widget_id}}}.step"
  :show-input="data.{{{widget.widget_id}}}.showInput"
  :show-input-controls="data.{{{widget.widget_id}}}.showInputControls"
  :show-stops="data.{{{widget.widget_id}}}.showStops"
  :show-tooltip="data.{{{widget.widget_id}}}.showTooltip"
  :range="data.{{{widget.widget_id}}}.range"
  :vertical="data.{{{widget.widget_id}}}.vertical"
  :height="data.{{{widget.widget_id}}}.height"
>
</el-slider>
